# -*- coding: utf-8 -*-

import unittest

import aqi.algos
import aqi.algos.epa
import aqi.constants


class TestAlgos(unittest.TestCase):
    """
    Test the aqi.algo module
    """

    def test_get_algo(self):
        """Test algo loading"""
        self.assertIsNone(
            aqi.algos.get_algo('qwertyuiop'))
        self.assertIsInstance(
            aqi.algos.get_algo(aqi.constants.ALGO_EPA),
            aqi.algos.epa.AQI)



# -*- coding: utf-8 -*-

from decimal import Decimal
import unittest

from aqi.algos.epa import AQI
from aqi.constants import (POLLUTANT_PM25, POLLUTANT_PM10,
                          POLLUTANT_O3_8H, POLLUTANT_O3_1H,
                          POLLUTANT_CO_8H, POLLUTANT_SO2_1H,
                          POLLUTANT_NO2_1H)


class TestEPA(unittest.TestCase):
    """
    Test the EPA algorithm based on examples from the EPA official doc.
    """

    def test_PM25(self):
        """Test PM2.5 AQI"""
        myaqi = AQI()
        self.assertEqual(
            myaqi.iaqi(POLLUTANT_PM25, '9.3'),
            39)
        self.assertEqual(
            myaqi.iaqi(POLLUTANT_PM25, '15'),
            57)
        self.assertEqual(
            myaqi.iaqi(POLLUTANT_PM25, '49.5'),
            135)
        self.assertEqual(
            myaqi.iaqi(POLLUTANT_PM25, '235.4'),
            285)

    def test_O3(self):
        """Test Ozone AQI"""
        myaqi = AQI()
        self.assertEqual(
            myaqi.iaqi(POLLUTANT_O3_8H, '0.08753333'),
            129)
        self.assertEqual(
            myaqi.iaqi(POLLUTANT_O3_1H, '0.162'),
            147)
        self.assertEqual(
            myaqi.iaqi(POLLUTANT_O3_8H, '0.141'),
            211)

    def test_O3_PM25_CO(self):
        """Test O3, PM2.5 and CO AQI"""
        myaqi = AQI()
        self.assertEqual(
            myaqi.iaqi(POLLUTANT_O3_8H, '0.077')
            , 104)
        self.assertEqual(
            myaqi.iaqi(POLLUTANT_PM25, '35.9')
            , 102)
        self.assertEqual(
            myaqi.iaqi(POLLUTANT_CO_8H, '8.4'),
            90)
        self.assertEqual(
            myaqi.aqi([
                (POLLUTANT_O3_8H, '0.077'),
                (POLLUTANT_PM25, '35.9'),
                (POLLUTANT_CO_8H, '8.4')
            ]),
            104)

    def test_to_cc(self):
        """Test conversion to concentration"""
        myaqi = AQI()
        self.assertEqual(
            myaqi.cc(POLLUTANT_PM25, '39'),
            Decimal('9.3'))
        self.assertEqual(
            myaqi.cc(POLLUTANT_PM25, '50'),
            Decimal('12'))
        self.assertEqual(
            myaqi.cc(POLLUTANT_PM25, '100'),
            Decimal('35.4'))
        self.assertEqual(
            myaqi.cc(POLLUTANT_PM25, '345'),
            Decimal('294.9'))

    def test_blank_bp(self):
        """Test blank breakpoints"""
        myaqi = AQI()
        self.assertEqual(
            myaqi.iaqi(POLLUTANT_O3_8H, '0.087'),
            129)



# -*- coding: utf-8 -*-

# constants for pollutants names
POLLUTANT_O3_8H = 'o3_8h'
POLLUTANT_O3_1H = 'o3_1h'
POLLUTANT_PM10 = 'pm10'
POLLUTANT_PM25 = 'pm25'
POLLUTANT_CO_1H = 'co_1h'
POLLUTANT_CO_8H = 'co_8h'
POLLUTANT_CO_24H = 'co_24h'
POLLUTANT_SO2_1H = 'so2_1h'
POLLUTANT_SO2_24H = 'so2_24h'
POLLUTANT_NO2_1H = 'no2_1h'
POLLUTANT_NO2_24H = 'no2_24h'

# constants for algorithms, canonical module name
ALGO_EPA = 'aqicalc.algos.epa'
ALGO_MEP = 'aqicalc.algos.mep'
ALGO_CPCB = 'aqicalc.algos.cpcb'
ALGO_IEMA = 'aqicalc.algos.iema'
ALGO_KAQI = 'aqicalc.algos.kaqi'
ALGO_CAQI = 'aqicalc.algos.caqi'
ALGOS = [ALGO_EPA, ALGO_MEP,ALGO_CPCB,ALGO_IEMA,ALGO_KAQI]



from decimal import *

from aqi.constants import *
from aqi.algos.base import PiecewiseAQI


class AQI(PiecewiseAQI):
    """Implementation of the CAQI algorithm.
    """

    piecewise = {
        'aqi': [
            (Decimal(0), Decimal(24.99)),
            (Decimal(25), Decimal(49.99)),
            (Decimal(50), Decimal(74.99)),
            (Decimal(75), Decimal(99.99)),
            (Decimal(100), Decimal(1000))],
        'bp': {
            POLLUTANT_NO2_1H: [
                (Decimal('0'), Decimal('49.99')),
                (Decimal('50'), Decimal('99.99')),
                (Decimal('100'), Decimal('199.99')),
                (Decimal('200'), Decimal('399.99')),
                (Decimal('400'), Decimal('inf')),
            ],
            POLLUTANT_O3_1H: [
                (Decimal('0.0'), Decimal('60.000')),
                (Decimal('60.001'), Decimal('120.000')),
                (Decimal('120.001'), Decimal('180.000')),
                (Decimal('180.001'), Decimal('240.000')),
                (Decimal('240.001'), Decimal('inf')),
            ],
            POLLUTANT_PM10: [
                (Decimal('0'), Decimal('25')),
                (Decimal('25.01'), Decimal('50')),
                (Decimal('50.01'), Decimal('90')),
                (Decimal('90.01'), Decimal('180')),
                (Decimal('180.01'), Decimal('inf')),
            ],
            POLLUTANT_PM25: [
                (Decimal('0.0'), Decimal('15.0')),
                (Decimal('15.01'), Decimal('30.0')),
                (Decimal('30.01'), Decimal('55.0')),
                (Decimal('55.01'), Decimal('110.0')),
                (Decimal('110.01'), Decimal('inf')),
            ],
            POLLUTANT_CO_8H: [
                (Decimal('0.0'), Decimal('5000.0')),
                (Decimal('5000.01'), Decimal('7500.0')),
                (Decimal('7500.01'), Decimal('10000.0')),
                (Decimal('10000.01'), Decimal('20000.0')),
                (Decimal('20000.01'), Decimal('inf')),
            ],
            POLLUTANT_SO2_1H: [
                (Decimal('0'), Decimal('50')),
                (Decimal('50.01'), Decimal('100')),
                (Decimal('100.01'), Decimal('350')),
                (Decimal('350.01'), Decimal('500')),
                (Decimal('500.01'), Decimal('inf')),
            ],
        },
        'prec': {
            POLLUTANT_O3_1H: Decimal('.001'),
            POLLUTANT_PM10: Decimal('.1'),
            POLLUTANT_PM25: Decimal('.1'),
            POLLUTANT_CO_8H: Decimal('.1'),
            POLLUTANT_SO2_1H: Decimal('1.'),
            POLLUTANT_NO2_1H: Decimal('1.'),
        },
        'units': {
            POLLUTANT_O3_1H: 'µg/m³',
            POLLUTANT_PM10: 'µg/m³',
            POLLUTANT_PM25: 'µg/m³',
            POLLUTANT_CO_8H: 'µg/m³',
            POLLUTANT_SO2_1H: 'µg/m³',
            POLLUTANT_NO2_1H: 'µg/m³',
        },
    }




# -*- coding: utf-8 -*-

from decimal import *


class BaseAQI(object):
    """A generic AQI class"""

    def iaqi(self, elem, cc):
        """Calculate an intermediate AQI for a given pollutant. This is
        the heart of the algo. Return the IAQI for the given pollutant.

        .. warning:: the concentration is passed as a string so
        :class:`decimal.Decimal` doesn't act up with binary floats.

        :param elem: pollutant constant
        :type elem: int
        :param cc: pollutant contentration (µg/m³ or ppm)
        :type cc: str
        """
        raise NotImplementedError

    def aqi(self, ccs, iaqis=False):
        """Calculate the AQI based on a list of pollutants. Return an
        AQI value, if `iaqis` is set to True, send back a tuple
        containing the AQI and a dict of IAQIs.

        :param ccs: a list of tuples of pollutants concentrations with
                    pollutant constant and concentration as values
        :type ccs: list
        :param iaqis: return IAQIs with result
        :type iaqis: bool
        """
        try:
            _iaqis = {}
            for (elem, cc) in ccs:
                _iaqi = self.iaqi(elem, cc)
                if _iaqi is not None:
                    _iaqis[elem] = _iaqi
            _aqi = max(_iaqis.values())
            if iaqis:
                return (_aqi, _iaqis)
            else:
                return _aqi
        except:
            return None

    def cc(self, elem, iaqi):
        """Calculate a concentration for a given pollutant. Return the
        concentration for the given pollutant based on the intermediate AQI.

        .. warning:: the intermediate AQI is passed as a string

        :param elem: pollutant constant
        :type elem: int
        :param cc: intermediate AQI
        :type cc: str
        """
        raise NotImplementedError

    def list_pollutants(self):
        """List pollutants covered by an algorithm, return a list of
        pollutant names.
        """
        raise NotImplementedError


class PiecewiseAQI(BaseAQI):
    """A piecewise function AQI class (like EPA or MEP)"""

    piecewise = None

    def iaqi(self, elem, cc):
        try:
            if self.piecewise is None:
                raise NameError("piecewise struct is not defined")
            if elem not in self.piecewise['bp'].keys():
                return None

            _cc = Decimal(cc).quantize(self.piecewise['prec'][elem],
                                    rounding=ROUND_DOWN)

            # define breakpoints for this pollutant at this contentration
            bps = self.piecewise['bp'][elem]
            bplo = None
            bphi = None
            idx = 0
            for bp in bps:
                if _cc >= bp[0] and _cc <= bp[1]:
                    bplo = bp[0]
                    bphi = bp[1]
                    break
                idx += 1
            # get corresponding AQI boundaries
            (aqilo, aqihi) = self.piecewise['aqi'][idx]

            # equation
            value = (aqihi - aqilo) / (bphi - bplo) * (_cc - bplo) + aqilo
            return value.quantize(Decimal('1.'), rounding=ROUND_HALF_EVEN)
        except:
            return cc


    def cc(self, elem, iaqi):
        try:
            if self.piecewise is None:
                raise NameError("piecewise struct is not defined")
            if elem not in self.piecewise['bp'].keys():
                return None

            _iaqi = int(iaqi)

            # define aqi breakpoints for this pollutant at this IAQI
            bps = self.piecewise['aqi']
            bplo = None
            bphi = None
            idx = 0
            for bp in bps:
                if _iaqi >= bp[0] and _iaqi <= bp[1]:
                    bplo = bp[0]
                    bphi = bp[1]
                    break
                idx += 1
            # get corresponding concentration boundaries
            (cclo, cchi) = self.piecewise['bp'][elem][idx]

            # equation
            value = (cchi - cclo) / (bphi - bplo) * (_iaqi - bplo) + cclo
            return Decimal(value).quantize(self.piecewise['prec'][elem],
                                    rounding=ROUND_DOWN)
        except:
            return iaqi

    def list_pollutants(self):
        return self.piecewise['units'].items()

    def get_units(self):
        return self.piecewise['units']


# -*- coding: utf-8 -*-

from decimal import *

from aqicalc.constants import (POLLUTANT_PM25, POLLUTANT_PM10,
                          POLLUTANT_O3_8H, POLLUTANT_O3_1H,
                          POLLUTANT_CO_1H, POLLUTANT_CO_24H,
                          POLLUTANT_SO2_1H, POLLUTANT_SO2_24H,
                          POLLUTANT_NO2_1H, POLLUTANT_NO2_24H)
from aqicalc.algos.base import PiecewiseAQI


class AQI(PiecewiseAQI):
    """Implementation of the MEP AQI algorithm.
    """

    piecewise = {
        'aqi': [
            (0, 50),
            (51, 100),
            (101, 150),
            (151, 200),
            (201, 300),
            (301, 400),
            (401, 500)],
        'bp': {
            POLLUTANT_SO2_24H: [
                (Decimal('0.0'), Decimal('50')),
                (Decimal('51'), Decimal('150')),
                (Decimal('151'), Decimal('475')),
                (Decimal('476'), Decimal('800')),
                (Decimal('801'), Decimal('1600')),
                (Decimal('1601'), Decimal('2100')),
                (Decimal('2101'), Decimal('2620')),
            ],
            POLLUTANT_SO2_1H: [
                (Decimal('0.0'), Decimal('150')),
                (Decimal('151'), Decimal('500')),
                (Decimal('501'), Decimal('650')),
                (Decimal('651'), Decimal('800')),
            ],
            POLLUTANT_NO2_24H: [
                (Decimal('0.0'), Decimal('40')),
                (Decimal('41'), Decimal('80')),
                (Decimal('81'), Decimal('180')),
                (Decimal('181'), Decimal('280')),
                (Decimal('281'), Decimal('565')),
                (Decimal('566'), Decimal('750')),
                (Decimal('751'), Decimal('940')),
            ],
            POLLUTANT_NO2_1H: [
                (Decimal('0.0'), Decimal('100')),
                (Decimal('101'), Decimal('200')),
                (Decimal('201'), Decimal('700')),
                (Decimal('701'), Decimal('1200')),
                (Decimal('1201'), Decimal('2340')),
                (Decimal('2341'), Decimal('3090')),
                (Decimal('3091'), Decimal('3840')),
            ],
            POLLUTANT_PM10: [
                (Decimal('00'), Decimal('50')),
                (Decimal('51'), Decimal('150')),
                (Decimal('151'), Decimal('250')),
                (Decimal('251'), Decimal('350')),
                (Decimal('351'), Decimal('420')),
                (Decimal('421'), Decimal('500')),
                (Decimal('501'), Decimal('600')),
            ],
            POLLUTANT_CO_24H: [
                (Decimal('0.0'), Decimal('2')),
                (Decimal('3'), Decimal('4')),
                (Decimal('5'), Decimal('14')),
                (Decimal('15'), Decimal('24')),
                (Decimal('25'), Decimal('36')),
                (Decimal('37'), Decimal('48')),
                (Decimal('49'), Decimal('60')),
            ],
            POLLUTANT_CO_1H: [
                (Decimal('0.0'), Decimal('5')),
                (Decimal('6'), Decimal('10')),
                (Decimal('11'), Decimal('35')),
                (Decimal('36'), Decimal('60')),
                (Decimal('61'), Decimal('90')),
                (Decimal('91'), Decimal('120')),
                (Decimal('121'), Decimal('150')),
            ],
            POLLUTANT_O3_1H: [
                (Decimal('0.0'), Decimal('160')),
                (Decimal('161'), Decimal('200')),
                (Decimal('201'), Decimal('300')),
                (Decimal('301'), Decimal('400')),
                (Decimal('401'), Decimal('800')),
                (Decimal('801'), Decimal('1000')),
                (Decimal('1001'), Decimal('1200')),
            ],
            POLLUTANT_O3_8H: [
                (Decimal('0.0'), Decimal('100')),
                (Decimal('101'), Decimal('160')),
                (Decimal('161'), Decimal('215')),
                (Decimal('216'), Decimal('265')),
                (Decimal('266'), Decimal('800')),
            ],
            POLLUTANT_PM25: [
                (Decimal('0.0'), Decimal('35')),
                (Decimal('36'), Decimal('75')),
                (Decimal('76'), Decimal('115')),
                (Decimal('116'), Decimal('150')),
                (Decimal('151'), Decimal('250')),
                (Decimal('251'), Decimal('350')),
                (Decimal('351'), Decimal('500')),
            ],
        },
        'prec': {
            POLLUTANT_O3_8H: Decimal('1.'),
            POLLUTANT_O3_1H: Decimal('1.'),
            POLLUTANT_PM10: Decimal('1.'),
            POLLUTANT_PM25: Decimal('1.'),
            POLLUTANT_CO_1H: Decimal('1.'),
            POLLUTANT_CO_24H: Decimal('1.'),
            POLLUTANT_SO2_1H: Decimal('1.'),
            POLLUTANT_SO2_24H: Decimal('1.'),
            POLLUTANT_NO2_1H: Decimal('1.'),
            POLLUTANT_NO2_24H: Decimal('1.'),
        },
        'units': {
            POLLUTANT_SO2_24H: 'µg/m³',
            POLLUTANT_SO2_1H: 'µg/m³',
            POLLUTANT_NO2_24H: 'µg/m³',
            POLLUTANT_NO2_1H: 'µg/m³',
            POLLUTANT_PM10: 'µg/m³',
            POLLUTANT_CO_24H: 'mg/m³',
            POLLUTANT_CO_1H: 'mg/m³',
            POLLUTANT_O3_1H: 'µg/m³',
            POLLUTANT_O3_8H: 'µg/m³',
            POLLUTANT_PM25: 'µg/m³',
        },
    }



# -*- coding: utf-8 -*-

from decimal import *

from aqicalc.constants import *
from aqicalc.algos.base import PiecewiseAQI
from math import inf

class AQI(PiecewiseAQI):
    """Implementation of the CPCB AQI algorithm, CO is in mg/m3 units is in ug/3 for all except CO.
    """

    piecewise = {
        'aqi': [
            (0, 40),
            (41, 80),
            (81, 120),
            (121, 200),
            (201, inf)],
        'bp': {
            POLLUTANT_O3_8H: [
                (Decimal('0'), Decimal('100')),
                (Decimal('101'), Decimal('130')),
                (Decimal('131'), Decimal('160')),
                (Decimal('161'), Decimal('200')),
                (Decimal('201'), inf),
            ],
            POLLUTANT_O3_1H: [
                (Decimal('0'), Decimal('100')),
                (Decimal('101'), Decimal('130')),
                (Decimal('131'), Decimal('160')),
                (Decimal('161'), Decimal('200')),
                (Decimal('201'), inf),
            ],
            POLLUTANT_PM10: [
                (Decimal('0'), Decimal('50')),
                (Decimal('51'), Decimal('100')),
                (Decimal('101'), Decimal('150')),
                (Decimal('151'), Decimal('250')),
                (Decimal('251'), inf),
            ],
            POLLUTANT_PM25: [
                (Decimal('0'), Decimal('25')),
                (Decimal('26'), Decimal('50')),
                (Decimal('51'), Decimal('75')),
                (Decimal('76'), Decimal('125')),
                (Decimal('126'), inf),
            ],
            POLLUTANT_CO_8H: [
                (Decimal('0.0'), Decimal('10.0')),
                (Decimal('10.1'), Decimal('13.0')),
                (Decimal('13.1'), Decimal('15.0')),
                (Decimal('15.1'), Decimal('17.0')),
                (Decimal('17.1'), inf),
            ],
            POLLUTANT_SO2_1H: [
                (Decimal('0'), Decimal('20')),
                (Decimal('21'), Decimal('40')),
                (Decimal('41'), Decimal('365')),
                (Decimal('366'), Decimal('800')),
                (Decimal('801'), inf),
            ],
            POLLUTANT_NO2_1H: [
                (Decimal('0'), Decimal('200')),
                (Decimal('201'), Decimal('240')),
                (Decimal('241'), Decimal('320')),
                (Decimal('321'), Decimal('1130')),
                (Decimal('1131'),inf),
            ],
        },
        'prec': {
            POLLUTANT_O3_8H: Decimal('.1'),
            POLLUTANT_O3_1H: Decimal('.1'),
            POLLUTANT_PM10: Decimal('1.'),
            POLLUTANT_PM25: Decimal('.1'),
            POLLUTANT_CO_8H: Decimal('.1'),
            POLLUTANT_SO2_1H: Decimal('1.'),
            POLLUTANT_NO2_1H: Decimal('1.'),
        },
        'units': {
            POLLUTANT_O3_8H: 'µg/m³',
            POLLUTANT_O3_1H: 'µg/m³',
            POLLUTANT_PM10: 'µg/m³',
            POLLUTANT_PM25: 'µg/m³',
            POLLUTANT_CO_8H: 'mg/m³',
            POLLUTANT_SO2_1H: 'µg/m³',
            POLLUTANT_NO2_1H: 'µg/m³',
        },
    }



# -*- coding: utf-8 -*-

from decimal import *

from aqicalc.constants import *
from aqicalc.algos.base import PiecewiseAQI
from math import inf

class AQI(PiecewiseAQI):
    """Implementation of the CPCB AQI algorithm, CO is in mg/m3 units is in ug/3 for all except CO.
    """

    piecewise = {
        'aqi': [
            (0, 50),
            (51, 100),
            (101, 200),
            (201, 300),
            (301, 400),
            (301, 400),
            (401, inf)],
        'bp': {
            POLLUTANT_O3_8H: [
                (Decimal('0'), Decimal('50')),
                (Decimal('51'), Decimal('100')),
                (Decimal('101'), Decimal('168')),
                (Decimal('169'), Decimal('208')),
                (Decimal('209'), Decimal('748')),
                (Decimal('749'), inf),
            ],
            POLLUTANT_O3_1H: [
                (Decimal('0'), Decimal('50')),
                (Decimal('51'), Decimal('100')),
                (Decimal('101'), Decimal('168')),
                (Decimal('169'), Decimal('208')),
                (Decimal('209'), Decimal('748')),
                (Decimal('749'), inf),
            ],
            POLLUTANT_PM10: [
                (Decimal('0'), Decimal('50')),
                (Decimal('51'), Decimal('100')),
                (Decimal('101'), Decimal('250')),
                (Decimal('251'), Decimal('350')),
                (Decimal('351'), Decimal('430')),
                (Decimal('431'), inf),
            ],
            POLLUTANT_PM25: [
                (Decimal('0'), Decimal('30')),
                (Decimal('31'), Decimal('60')),
                (Decimal('61'), Decimal('90')),
                (Decimal('91'), Decimal('120')),
                (Decimal('121'), Decimal('250')),
                (Decimal('251'), inf),
            ],
            POLLUTANT_CO_8H: [
                (Decimal('0.0'), Decimal('1.0')),
                (Decimal('1.1'), Decimal('2.0')),
                (Decimal('2.1'), Decimal('10.0')),
                (Decimal('10.1'), Decimal('17.0')),
                (Decimal('17.1'), Decimal('34.0')),
                (Decimal('34.1'), inf),
            ],
            POLLUTANT_SO2_1H: [
                (Decimal('0'), Decimal('40')),
                (Decimal('41'), Decimal('80')),
                (Decimal('81'), Decimal('380')),
                (Decimal('381'), Decimal('800')),
                (Decimal('801'), Decimal('1600')),
                (Decimal('1601'), inf),
            ],
            POLLUTANT_NO2_1H: [
                (Decimal('0'), Decimal('40')),
                (Decimal('41'), Decimal('80')),
                (Decimal('81'), Decimal('180')),
                (Decimal('181'), Decimal('280')),
                (Decimal('281'), Decimal('400')),
                (Decimal('401'), inf),
            ],
        },
        'prec': {
            POLLUTANT_O3_8H: Decimal('.1'),
            POLLUTANT_O3_1H: Decimal('.1'),
            POLLUTANT_PM10: Decimal('1.'),
            POLLUTANT_PM25: Decimal('.1'),
            POLLUTANT_CO_8H: Decimal('.1'),
            POLLUTANT_SO2_1H: Decimal('1.'),
            POLLUTANT_NO2_1H: Decimal('1.'),
        },
        'units': {
            POLLUTANT_O3_8H: 'µg/m³',
            POLLUTANT_O3_1H: 'µg/m³',
            POLLUTANT_PM10: 'µg/m³',
            POLLUTANT_PM25: 'µg/m³',
            POLLUTANT_CO_8H: 'mg/m³',
            POLLUTANT_SO2_1H: 'µg/m³',
            POLLUTANT_NO2_1H: 'µg/m³',
        },
    }



# -*- coding: utf-8 -*-
import pkgutil
from aqicalc.constants import ALGOS

def get_algo(algo_mod):
    """Instanciate an AQI algorithm class. If there is a problem during
    the import or instanciation, return None.

    :param algo_mod: algorithm module canonical name
    :type algo_mod: str
    """
    # try to import the algorithm module
    try:
        mod = __import__(algo_mod, fromlist=[algo_mod])
    except ImportError:
        return None

    # try to instanciate an AQI class
    try:
        return mod.AQI()
    except AttributeError:
        return None

def list_algos():
    """Return a list of available algorithms with corresponding
    pollutant
    """
    return ALGOS



# -*- coding: utf-8 -*-

from aqicalc.constants import *

from aqicalc.algos import get_algo, list_algos

__author__ = "Stefan \"hr\" Berder"
__contact__ = "hr@bonz.org"
__license__ = "BSD 3-Clause"
__version__ = "0.6.1"


class InvalidConversionArguments(Exception):
    pass


def to_iaqi(elem, cc, algo=ALGO_EPA):
    """Calculate an intermediate AQI for a given pollutant. This is the
    heart of the algo.

    .. warning:: the concentration is passed as a string so
        :class:`decimal.Decimal` doesn't act up with binary floats.

    :param elem: pollutant constant
    :type elem: int
    :param cc: pollutant contentration (µg/m³ or ppm)
    :type cc: str
    :param algo: algorithm module canonical name
    :type algo: str
    """
    _aqi = get_algo(algo)
    return _aqi.iaqi(elem, cc)


def to_aqi(ccs, algo=ALGO_EPA):
    """Calculate the AQI based on a list of pollutants

    :param ccs: a list of tuples of pollutants concentrations with
                pollutant constant and concentration as values
    :type ccs: list
    :param algo: algorithm module name
    :type algo: str
    """
    _aqi = get_algo(algo)
    return _aqi.aqi(ccs)


def to_cc(elem, iaqi, algo=ALGO_EPA):
    """Calculate a concentration for a given pollutant.

    .. warning:: the intermediate AQI is passed as a string

    :param elem: pollutant constant
    :type elem: int
    :param cc: intermediate AQI
    :type iaqi: str
    :param algo: algorithm module canonical name
    :type algo: str
    """
    _aqi = get_algo(algo)
    return _aqi.cc(elem, iaqi)


def get_units(algo=ALGO_EPA):
    """
    Returns units
    """
    _aqi = get_algo(algo)
    return _aqi.get_units()

def convert_grams_to_parts(x, pollutant):
    """
    Converts ug/m3 to ppb for O3, NO2 and SO2 and mg/m3 to ppm for CO.
    Note: If you are looking to convert ug/m3 for CO, multiply the results by 1000
    If you are looking to convert ppm to ppb multiply results by 1000 and divide for vice versa
    """
    if pollutant == POLLUTANT_O3_1H or pollutant == POLLUTANT_O3_8H:
        return x/1.96  # ugm3 to ppb
    if pollutant == POLLUTANT_NO2_1H or pollutant == POLLUTANT_NO2_24H:
        return x/1.88  # ugm3 to ppb
    if pollutant == POLLUTANT_CO_1H or pollutant == POLLUTANT_CO_24H or pollutant == POLLUTANT_CO_8H:
        return x/1.15  # mgm3 to ppb
    if pollutant == POLLUTANT_SO2_1H or pollutant == POLLUTANT_SO2_24H:
        return x/2.62  # ugm3 to ppb
    else:
        raise InvalidConversionArguments('Pollutant or Value Invalid')

def convert_ppm_to_ppb(ppm):
    "converts ppm to ppb"
    return ppm*1000

def convert_ppb_to_ppm(ppb):
    "converts ppb to ppm"
    return ppb/1000

def convert_pphm_to_ppb(pphm):
    "converts pphm to ppb"
    return pphm*10

def convert_ppb_to_pphm(ppb):
    "converts ppb to pphm"
    return ppb/10

def convert_pphm_to_ppm(pphm):
    "converts pphm to ppm"
    return pphm/100

def convert_ppm_to_pphm(ppm):
    "converts ppm to pphm"
    return ppm*100

def convert_milli_to_micro(milli):
    "converts mg/m3 to ug/m3"
    return milli*1000

def convert_micro_to_milli(micro):
    "converts ug/m3 to mg/m3"
    return micro/1000

def convert_parts_to_grams(x, pollutant):
    """
    Converts ppb to ug/m3 for O3, NO2 and SO2 and ppm to mg/m3 for CO.
    Note: If you are looking to convert ppb for CO, multiply the results by 1000
    If you are looking to convert ppm to ppb multiply results by 1000 and divide for vice versa
    """
    if pollutant == POLLUTANT_O3_1H or pollutant == POLLUTANT_O3_8H:
        return x*1.96  # ugm3 to ppb
    if pollutant == POLLUTANT_NO2_1H or pollutant == POLLUTANT_NO2_24H:
        return x*1.88  # ugm3 to ppb
    if pollutant == POLLUTANT_CO_1H or pollutant == POLLUTANT_CO_24H or pollutant == POLLUTANT_CO_8H:
        return x*1.15  # mgm3 to ppb
    if pollutant == POLLUTANT_SO2_1H or pollutant == POLLUTANT_SO2_24H:
        return x*2.62  # ugm3 to ppb
    else:
        raise InvalidConversionArguments('Pollutant or Value Invalid')


def console_aqi():
    """Console entry point, this function is used as an entry point to
    the 'aqi' command.
    """
    import sys
    import argparse

    parser = argparse.ArgumentParser(description="""Convert between AQI value and pollutant concentration (µg/m³ or
    ppm).""")
    parser.add_argument('-c', dest='conv', choices=['aqi', 'cc'], default='aqi',
                        help="Conversion to perform, defaults to 'aqi'")
    parser.add_argument('-l', action='store_true', dest='list',
                        help='list the available algorithms and '
                             'corresponding pollutants')
    parser.add_argument('-v', action='store_true', dest='verbose',
                        help='add IAQIs to the result')
    parser.add_argument('algo', nargs='?',
                        help='the formula to use for the AQI '
                             'calculation, use the python module path')
    parser.add_argument('measures', nargs='*', metavar='measure',
                        help='pollutant measure, format is '
                             'element_name:concentration. Unknown '
                             'pollutants are silently ignored.')
    args = parser.parse_args()

    # list available algorithms
    if args.list is True:
        for _algo in list_algos():
            print("{algo}: {elem}".format(
                algo=_algo[0], elem=', '.join(
                    ["{0} ({1})".format(elem, unit) for (elem, unit)
                     in _algo[1]])))
    else:
        # if not listing but missing other positional argument
        if args.algo is None or args.measures is None:
            sys.stderr.write("Missing algorithm or measure.\n")
            parser.print_help()
            sys.exit(1)
        _aqi = get_algo(args.algo)
        # couln't load the algo module or instanciate AQI class
        if _aqi is None:
            sys.stderr.write("Unknown algorithm or module is missing an "
                             "AQI class\n")
            parser.print_help()
            sys.exit(1)
        if args.conv == 'aqi':
            ccs = []
            for measure in args.measures:
                (elem, cc) = measure.split(':')
                ccs.append((elem, cc))

            ret = _aqi.aqi(ccs, iaqis=args.verbose)
            if args.verbose is True:
                iaqis = []
                for (constant, iaqi) in ret[1].items():
                    iaqis.append(constant + ':' + str(iaqi))
                sys.stdout.write(' '.join(iaqis) + "\n")
                sys.stdout.write(str(ret[0]) + "\n")
            else:
                sys.stdout.write(str(ret) + "\n")
        else:
            iaqis = []
            for measure in args.measures:
                (elem, iaqi) = measure.split(':')
                iaqis.append((elem, iaqi))

            ret = []
            for iaqi in iaqis:
                ret.append((iaqi[0], _aqi.cc(iaqi[0], iaqi[1])))
            if len(ret) == 1:
                sys.stdout.write(str(ret[0][1]) + "\n")
            elif len(ret) > 1:
                ccs = []
                for (elem, cc) in ret:
                    if cc is None:
                        ccs.append(elem + ':na')
                    else:
                        ccs.append(elem + ':' + str(cc))
                sys.stdout.write('\n'.join(ccs) + "\n")

    # end the script without a problem
    sys.exit(0)



# -*- coding: utf-8 -*-

import unittest

import aqi.algos.base


class TestBaseAQI(unittest.TestCase):
    """
    Test the base module
    """

    def test_aqi(self):
        """Test the BaseAQI class"""
        _aqi = aqi.algos.base.BaseAQI()
        # dummy iaqi function, return the concentration
        _aqi.iaqi = lambda x, y: int(y)
        self.assertEqual(
            _aqi.aqi([(0, '20'), (1,'30'), (2, '100')]),
            100)
        self.assertEqual(
            _aqi.aqi([(0, '20'), (1,'30'), (2, '100')], iaqis=True),
            (100, {0: 20, 1: 30, 2: 100}))



# -*- coding: utf-8 -*-
import os
from setuptools import setup, find_packages


here = os.path.abspath(os.path.dirname(__file__))

long_description = open(os.path.join(here, 'README.rst')).read()

# allow setup.py to be run from any path
#os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='aqicalc',
    version="0.0.7",
    author="Pareekshith US Katti",
    author_email=" ",
    license="BSD 3-Clause",
    packages=find_packages(exclude=['contrib', 'docs', 'test*']),
    url='https://github.com/Paree24/aqicalc',
    description='A library to convert between AQI value and pollutant '
    'concentration (µg/m³ or ppm)',
    long_description=long_description,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: Healthcare Industry',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python :: 3.9',
    ],
    keywords='air quality pm2.5 EPA MEP CPCB',
    entry_points = {
        'console_scripts': [
            'aqi=aqi:console_aqi',
        ],
    },
)



# -*- coding: utf-8 -*-

import unittest

import aqi


class TestAqi(unittest.TestCase):
    """
    Test the aqi module with EPA formula
    """

    def test_to_iaqi(self):
        """Test IAQI conversion"""
        self.assertEqual(
            aqi.to_iaqi(aqi.POLLUTANT_O3_8H, '0.08753333', algo=aqi.ALGO_EPA),
            129)

    def test_to_aqi(self):
        """Test AQI conversion"""
        self.assertEqual(
            aqi.to_aqi([
                (aqi.POLLUTANT_O3_8H, '0.077'),
                (aqi.POLLUTANT_PM25, '35.9'),
                (aqi.POLLUTANT_CO_8H, '8.4')
            ],
            algo=aqi.ALGO_EPA),
            104)



# -*- coding: utf-8 -*-

from decimal import *

from aqicalc.constants import (POLLUTANT_PM25, POLLUTANT_PM10,
                          POLLUTANT_O3_8H, POLLUTANT_O3_1H,
                          POLLUTANT_CO_8H, POLLUTANT_SO2_1H,
                          POLLUTANT_NO2_1H)
from aqicalc.algos.base import PiecewiseAQI
from math import inf

class AQI(PiecewiseAQI):
    """Implementation of the EPA AQI algorithm.
    """

    piecewise = {
        'aqi': [
            (0, 50),
            (51, 100),
            (101, 150),
            (151, 200),
            (201, 300),
            (301, 500),
            ],
        'bp': {
            POLLUTANT_O3_8H: [
                (Decimal('0.000'), Decimal('0.054')),
                (Decimal('0.055'), Decimal('0.070')),
                (Decimal('0.071'), Decimal('0.085')),
                (Decimal('0.086'), Decimal('0.105')),
                (Decimal('0.106'), Decimal('0.2')),
                (Decimal('0.21'), inf)
            ],
            POLLUTANT_O3_1H: [
                (0, 0),
                (0, 0),
                (Decimal('0.125'), Decimal('0.164')),
                (Decimal('0.165'), Decimal('0.204')),
                (Decimal('0.205'), Decimal('0.404')),
                (Decimal('0.405'), Decimal('0.604')),
            ],
            POLLUTANT_PM10: [
                (Decimal('0'), Decimal('100')),
                (Decimal('100.1'), Decimal('350')),
                (Decimal('350.1'), Decimal('400')),
                (Decimal('400.1'), Decimal('450')),
                (Decimal('450.1'), Decimal('500')),
                (Decimal('500.1'), inf),
            ],
            POLLUTANT_PM25: [
                (Decimal('0.0'), Decimal('25.0')),
                (Decimal('25.1'), Decimal('75.0')),
                (Decimal('75.1'), Decimal('100')),
                (Decimal('100.1'), Decimal('150.4')),
                (Decimal('150.5'), Decimal('250.4')),
                (Decimal('250.5'), inf),
            ],
            POLLUTANT_CO_8H: [
                (Decimal('0.0'), Decimal('4.4')),
                (Decimal('4.5'), Decimal('9.4')),
                (Decimal('9.5'), Decimal('12.4')),
                (Decimal('12.5'), Decimal('15.4')),
                (Decimal('15.5'), Decimal('30.4')),
                (Decimal('30.5'), inf),
            ],
            POLLUTANT_SO2_1H: [
                (Decimal('0'), Decimal('35')),
                (Decimal('36'), Decimal('75')),
                (Decimal('76'), Decimal('185')),
                (Decimal('186'), Decimal('304')),
                (Decimal('305'), Decimal('604')),
                (Decimal('605'), inf),
            ],
            POLLUTANT_NO2_1H: [
                (Decimal('0'), Decimal('53')),
                (Decimal('54'), Decimal('100')),
                (Decimal('101'), Decimal('360')),
                (Decimal('361'), Decimal('649')),
                (Decimal('650'), Decimal('1249')),
                (Decimal('1250'), inf),
            ],
        },
        'prec': {
            POLLUTANT_O3_8H: Decimal('.001'),
            POLLUTANT_O3_1H: Decimal('.001'),
            POLLUTANT_PM10: Decimal('1.'),
            POLLUTANT_PM25: Decimal('.1'),
            POLLUTANT_CO_8H: Decimal('.1'),
            POLLUTANT_SO2_1H: Decimal('1.'),
            POLLUTANT_NO2_1H: Decimal('1.'),
        },
        'units': {
            POLLUTANT_O3_8H: 'ppm',
            POLLUTANT_O3_1H: 'ppm',
            POLLUTANT_PM10: 'µg/m³',
            POLLUTANT_PM25: 'µg/m³',
            POLLUTANT_CO_8H: 'ppm',
            POLLUTANT_SO2_1H: 'ppb',
            POLLUTANT_NO2_1H: 'ppb',
        },
    }
